#ifdef ESP8266
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
ESP8266WebServer server(80);
#else
#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>
#include <ESPmDNS.h>
WebServer server(80);
#endif

#include "EEPROM.h"

#define EEPROM_SIZE 36

//const char* ssid = "JESUITESFP_P5";
//const char* password = "internetcoses";

const char* ssid = "JESUITESFP_AP";
const char* password = "internetcoses";

#ifdef LED_BUILTIN
const int led = LED_BUILTIN;
#else
const int led = 13;
#endif

String szDad, szNsk, szAsk;

void handleRoot() {
  digitalWrite(led, 1);
#ifdef ESP8266
  server.send(200, "text/plain", "hello from esp8266!");
#else
  String content = "<html>\n<body>\n<form action='/checkValues'>";
  content += "\tDAd: <input type='text' name='dad' value=" + szDad + ">  (Device Address)<br>\n";
  content += "\tNSK: <input type='text' name='nsk' value=" + szNsk + ">  (Network Session Key)<br>\n";
  content += "\tASK: <input type='text' name='ask' value=" + szAsk + ">  (App Session Key)<br>\n";
  content += "\t<input type='submit' value='Submit'>\n";
  content += "</form>\n";
  content += "\n<BR><a href='/rdCredentials'>Read current credentials</a>\n<BR>";
  content += "</body>\n</html>\n";

  server.send(200, "text/html", content);
#endif
  digitalWrite(led, 0);
}

void handleNotFound() {
  digitalWrite(led, 1);
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
  digitalWrite(led, 0);
}

void checkValues() {
  bool bOk = true;
  uint8_t i;
  //String message = "<html>\n<body>\nArguments: ";
  String message = "<html>\n<body>\n";

  //message += server.args();
  message += "<BR><BR>\n";
  for (i = 0; i < server.args(); i++) {
    //message += " " + server.argName(i) + ": " + server.arg(i) + "\n<BR>";
    if (server.argName(i) == "dad") {
      szDad = server.arg(i);
      message += "Device Address: " + szDad + "\n<BR>";
    }
    if (server.argName(i) == "nsk") {
      szNsk = server.arg(i);
      message += "Network Session Key: " + szNsk + "\n<BR>";
    }
    if (server.argName(i) == "ask") {
      szAsk = server.arg(i);
      message += "App Session Key: " + szAsk + "\n<BR>";
    }
  }
  if (szDad.length() != 8) {
    bOk = false;
    message += "\n<BR>Device address should be 8 symbols length<BR>\n";
  } else {
    for (i = 0; i < szDad.length() ; i++) {
      if ( !( (szDad.charAt(i) >= '0' && szDad.charAt(i) <= '9')
              || (szDad.charAt(i) >= 'A' && szDad.charAt(i) <= 'F')
              || (szDad.charAt(i) >= 'a' && szDad.charAt(i) <= 'f')
            )) {
        bOk = false;
        message += "\n<BR>Device address should content hexadecimal numbers only<BR>\n";
        break;
      }
    }
  }
  if (szNsk.length() != 32) {
    bOk = false;
    message += "\n<BR>Network Session Key should be 32 symbols length<BR>\n";
  } else {
    for (i = 0; i < szNsk.length() ; i++) {
      if ( !( (szNsk.charAt(i) >= '0' && szNsk.charAt(i) <= '9')
              || (szNsk.charAt(i) >= 'A' && szNsk.charAt(i) <= 'F')
              || (szNsk.charAt(i) >= 'a' && szNsk.charAt(i) <= 'f')
            )) {
        bOk = false;
        message += "\n<BR>Network Session Key should content hexadecimal numbers only<BR>\n";
        break;
      }
    }
  }
  if (szAsk.length() != 32) {
    bOk = false;
    message += "\n<BR>Application Session Key should be 32 symbols length<BR>\n";
  } else {
    for (i = 0; i < szAsk.length() ; i++) {
      if ( !( (szAsk.charAt(i) >= '0' && szAsk.charAt(i) <= '9')
              || (szAsk.charAt(i) >= 'A' && szAsk.charAt(i) <= 'F')
              || (szAsk.charAt(i) >= 'a' && szAsk.charAt(i) <= 'f')
            )) {
        bOk = false;
        message += "\n<BR>Application Session Key should content hexadecimal numbers only<BR>\n";
        break;
      }
    }
  }
  if (bOk) {
    message += "\n<BR><a href='/wrCredentials'>Write credentials</a><BR>\n";
    message += "\n<BR><a href='/'>Home page</a>\n</body>\n</html>";
  } else
    message += "\n<BR><a href='/'>Come back</a>\n</body>\n</html>";


  server.send(200, "text/html", message);
}

void rdCredentials() {
  String message = "<html>\n<body>\n";

  message += "<BR><BR>\n";
  message += "Device Address: " + szDad + "\n<BR>";
  message += "Network Session Key: " + szNsk + "\n<BR>";
  message += "App Session Key: " + szAsk + "\n<BR><BR>";
  message += "\n<BR><a href='/'>Home page</a>\n</body>\n</html>";

  server.send(200, "text/html", message);
}

int x2i(char *s)
{
  int x = 0;
  for (;;) {
    char c = *s;
    if (c >= '0' && c <= '9') {
      x *= 16;
      x += c - '0';
    }
    else if (c >= 'A' && c <= 'F') {
      x *= 16;
      x += (c - 'A') + 10;
    }
    else break;
    s++;
  }
  return x;
}

String wrCredentials() {
  int i, nPos;
  byte bBy;
  String szSub;
  char szC[5] = {0};

  Serial.println("<wrCred");
  Serial.print("szDad: ");
  Serial.println(szDad);
      nPos = 0;
  for ( i = 0; i < 4; i++) {
    szSub = szDad.substring(i * 2, i * 2 + 2);
    szSub.toCharArray(szC, 3);
    bBy = x2i(szC);/*
    Serial.print("szSub: ");
    Serial.println(szSub);
    Serial.print("szC: ");
    Serial.println(szC);
    Serial.print("bBy: ");*/

    EEPROM.write(nPos++, bBy);
    EEPROM.commit();
    Serial.print(bBy, 16);
    Serial.print(" ");
    delay(10);
  }
  Serial.println(" ");
  for (i = 0 ; i < 16; i++) {
    szSub = szNsk.substring(i * 2, i * 2 + 2);
    szSub.toCharArray(szC, 3);
    bBy = x2i(szC);
    EEPROM.write(nPos++, bBy);
    EEPROM.commit();
    Serial.print(bBy, 16);
    Serial.print(" ");
    delay(10);
  }
  Serial.println(" ");
  for (i = 0 ; i < 16; i++) {
    szSub = szAsk.substring(i * 2, i * 2 + 2);
    szSub.toCharArray(szC, 3);
    bBy = x2i(szC);
    EEPROM.write(nPos++, bBy);
    EEPROM.commit();
    Serial.print(bBy, 16);
    Serial.print(" ");
    delay(10);
  }
  Serial.println(" ");
  Serial.println();
  Serial.println("/wrCred>");
  String message = "<html>\n<body>\n";

  message += "writing credentials<BR><BR>\n\n";
  message += "szDad: " + szDad + "<BR>\n";
  message += "szNsk: " + szNsk + "<BR>\n";
  message += "szAsk: " + szAsk + "<BR>\n";
  message += "\n<BR><a href='/rdCredentials'>Read current credentials</a>\n<BR>";
  message += "\n<BR><a href='/'>Home page</a>\n</body>\n</html>";

  return message;
}

void rdEeprom() {
  int i;
  String szB, message = "<html>\n<body>\n";

  message += "<BR><BR>\n";

  Serial.println("start reading EEPROM...");
  if (!EEPROM.begin(EEPROM_SIZE))
  {
    Serial.println("failed to initialise EEPROM"); delay(1000000);
  }
  Serial.println("Device Address:");
  szDad = "";
  for ( i = 0; i < 4; i++)
  {
    szB = String(byte(EEPROM.read(i)), HEX);
    if (szB.length() == 1)
      szB = "0" + szB;
    szDad += szB;
    Serial.print(szB); Serial.print(" ");
  }
  Serial.println();
  Serial.print("szDad: ");
  Serial.println(szDad);
  //szDad = "00000000";
  //szNsk = "00000000000000000000000000000000";
  //szAsk = "00000000000000000000000000000000";
  Serial.println("Network Session Key:");
  szNsk = "";
  for ( ; i < 4 + 16; i++)
  {
    szB = String(byte(EEPROM.read(i)), HEX);
    if (szB.length() == 1)
      szB = "0" + szB;
    szNsk += szB;
    Serial.print(szB); Serial.print(" ");
  }
  Serial.println();
  Serial.print("szNsk: ");
  Serial.println(szNsk);
  Serial.println("App Session Key:");
  szAsk = "";
  for ( ; i < 4 + 16 + 16; i++)
  {
    szB = String(byte(EEPROM.read(i)), HEX);
    if (szB.length() == 1)
      szB = "0" + szB;
    szAsk += szB;
    Serial.print(szB); Serial.print(" ");
  }
  Serial.println();
  Serial.print("szAsk: ");
  Serial.println(szAsk);
  Serial.println("stop reading EEPROM...");
  /*
    message += "Device Address: " + szDad + "\n<BR>";
    message += "Network Session Key: " + szNsk + "\n<BR>";
    message += "App Session Key: " + szAsk + "\n<BR><BR>";
  */
  message += "\n<BR><a href='/'>Home page</a>\n</body>\n</html>";

  server.send(200, "text/html", message);
}

void setup(void) {
  pinMode(led, OUTPUT);
  digitalWrite(led, 0);
  Serial.begin(115200);

  rdEeprom(); // updates szDad, szNsk & szAsk
  /*
    WiFi.begin(ssid, password);
    Serial.println("");

    // Wait for connection
    while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    }
    Serial.println("");
    Serial.print("Connected to ");
    Serial.println(ssid);
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
  */
  WiFi.softAP(ssid, password);

  IPAddress IP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(IP);

  server.begin();
  /*
    #ifdef ESP8266
    if (MDNS.begin("esp8266")) {
    #else
    if (MDNS.begin("esp32")) {
    #endif
      Serial.println("MDNS responder started");
    }
  */
  server.on("/", handleRoot);
  server.on("/checkValues", checkValues);

  server.on("/inline", []() {
    server.send(200, "text/plain", "this works as well");
  });
  server.on("/wrCredentials", []() {
    String sz = wrCredentials();

    server.send(200, "text/html", sz);
  });
  server.on("/rdCredentials", rdCredentials);

  server.onNotFound(handleNotFound);

  server.begin();
  Serial.println("HTTP server started");
}

void loop(void) {
  server.handleClient();
}
